import ReactDOM from 'react-dom';

import InfoTpl from 'ui-component/cards/InfoTpl';
import viirsConfig from './viirsConfig';
import { getActiveTypeFilters, getActiveCategoryFilters } from 'utils/filterUtils';
import {
  deleteExtraPolygons,
  getPolygonFeatures,
  handleMarker,
  updateVNFFilter,
  popup,
  generateLayerConfig,
  getStyle,
  emitterClickHandler
} from 'utils/layerUtils';

const viirs = {
  config: viirsConfig,
  id: viirsConfig.id,
  name: viirsConfig.name,
  description: viirsConfig.description,
  enabled: viirsConfig.enabled,
  metadata: viirsConfig.metadata,
  requiredAuthLevel: viirsConfig.requiredAuthLevel,
  tooltipText: viirsConfig.tooltipText,
  on: {
    load: (map, draw, callback, stateObj) => [
      () => {
        const { layerState, filterState } = stateObj;
        const { circleRadius, circleColor } = getStyle(layerState.style, viirsConfig);
        if (map.getSource(viirsConfig.id) === undefined) {
          map.addSource(viirsConfig.id, viirsConfig.layerSource);
        }
        if (map.getLayer(viirsConfig.id) === undefined) {
          map.addLayer(
            generateLayerConfig(
              viirsConfig.id,
              viirsConfig.layerType,
              viirsConfig.paint(circleRadius, circleColor),
              viirsConfig.id,
              viirsConfig.sourceLayer,
              viirsConfig.metadata
            )
          );

          let countryFilter = null;
          const { selectedCountry } = filterState;
          if (filterState && selectedCountry && selectedCountry.alpha3) {
            countryFilter = ['==', 'iso', selectedCountry.alpha3];
          }

          let typeFilter = null;
          const activeTypeFilters = getActiveTypeFilters(filterState);
          if (activeTypeFilters.length) {
            typeFilter = ['in', 'type', ...activeTypeFilters];
          }

          let categoryFilter = null;
          const activeCategoryFilters = getActiveCategoryFilters(filterState);
          if (activeCategoryFilters.length) {
            categoryFilter = ['in', 'category', ...activeCategoryFilters];
          }

          const layersFilter = ['all'];
          if (countryFilter || typeFilter || categoryFilter) {
            if (countryFilter) {
              layersFilter.push(countryFilter);
            }
            if (typeFilter) {
              layersFilter.push(typeFilter);
            }
            if (categoryFilter) {
              layersFilter.push(categoryFilter);
            }
          }

          if (layersFilter.length > 1 && map.getLayer(viirsConfig.id)) {
            map.setFilter(viirsConfig.id, layersFilter);
          }
        }
      }
    ],
    click: (map, draw, callback) => [
      viirsConfig.id,
      (event) => emitterClickHandler(event, viirsConfig, map, draw, callback)
    ],
    mouseenter: (map, draw, callback, stateObj) => [
      viirsConfig.id,
      (event) => {
        const { properties } = event.features[0];
        const { lat, lon } = properties;
        const { id: layerId } = viirsConfig;
        map.getCanvas().style.cursor = 'pointer';
        const coordinates = [lon, lat].slice();
        if (properties) {
          const placeHolder = document.createElement('div');
          ReactDOM.render(
            <InfoTpl
              properties={{ ...properties, layerId }}
              countries={stateObj.filterState.countries}
            />,
            placeHolder
          );
          popup.setDOMContent(placeHolder).setLngLat(coordinates).addTo(map);
        }
      }
    ],
    mouseleave: (map) => [
      viirsConfig.id,
      () => {
        map.getCanvas().style.cursor = '';
        popup.remove();
      }
    ],
    'draw.create': (map, draw, callback, stateObj, dispatch, setSelectionToolCoordinates) => [
      (event) => {
        handleMarker(event, map, viirsConfig, draw);
        const featureCollection = deleteExtraPolygons(draw);
        if (featureCollection) {
          updateVNFFilter(
            map,
            viirsConfig.id,
            draw.getAll(),
            viirsConfig.tileSource,
            dispatch,
            setSelectionToolCoordinates
          );
        }
      }
    ],
    'draw.update': (map, draw, callback, stateObj, dispatch, setSelectionToolCoordinates) => [
      () => {
        const featureCollection = getPolygonFeatures(draw);
        if (featureCollection) {
          updateVNFFilter(
            map,
            viirsConfig.id,
            draw.getAll(),
            viirsConfig.tileSource,
            dispatch,
            setSelectionToolCoordinates
          );
        } else {
          map.getSource(viirsConfig.id).setTiles([viirsConfig.tileSource]);
          dispatch(setSelectionToolCoordinates(null));
        }
      }
    ],
    'draw.delete': (map, draw, callback, stateObj, dispatch, setSelectionToolCoordinates) => [
      () => {
        map.getSource(viirsConfig.id).setTiles([viirsConfig.tileSource]);
        dispatch(setSelectionToolCoordinates(null));
      }
    ]
  }
};

export default viirs;
