import { useEffect, useRef, useState } from 'react';
import { useTheme } from '@mui/material/styles';
import {
  Box,
  Chip,
  ClickAwayListener,
  Divider,
  List,
  ListItemButton,
  ListItemIcon,
  ListItemText,
  Paper,
  Popper,
  Stack,
  Typography
} from '@mui/material';

import MainCard from 'ui-component/cards/MainCard';
import useConfig from 'hooks/useConfig';
import LoginModal from 'ui-component/modals/LoginModal';
import Transitions from 'ui-component/extended/Transitions';
import { logout } from 'store/slices/user';
import { openSnackbar } from 'store/slices/snackbar';
import { dispatch, useSelector } from 'store';

import { IconLogout, IconSettings } from '@tabler/icons';

const MainMenu = () => {
  const theme = useTheme();
  const anchorRef = useRef(null);
  const userState = useSelector((state) => state.user);
  const [open, setOpen] = useState(false);
  const { borderRadius } = useConfig();

  const handleLogout = () => {
    dispatch(logout());
    dispatch(
      openSnackbar({
        anchorOrigin: {
          vertical: 'top',
          horizontal: 'center'
        },
        open: true,
        message: 'You are logged out',
        variant: 'alert',
        alert: {
          color: 'success'
        },
        close: true
      })
    );
  };

  const handleClose = (event) => {
    if (anchorRef.current && anchorRef.current.contains(event.target)) {
      return;
    }
    setOpen(false);
  };

  const handleToggle = () => {
    setOpen((prevOpen) => !prevOpen);
  };

  const prevOpen = useRef(open);
  useEffect(() => {
    if (prevOpen.current === true && open === false) {
      anchorRef.current.focus();
    }
    prevOpen.current = open;
  }, [open]);

  return (
    <>
      <Chip
        sx={{
          height: '48px',
          alignItems: 'center',
          borderRadius: '27px',
          transition: 'all .2s ease-in-out',
          borderColor:
            theme.palette.mode === 'dark' ? theme.palette.dark.main : theme.palette.primary.light,
          backgroundColor:
            theme.palette.mode === 'dark' ? theme.palette.dark.main : theme.palette.primary.light,
          '&[aria-controls="menu-list-grow"], &:hover': {
            borderColor: theme.palette.primary.main,
            background: `${theme.palette.primary.main}!important`,
            color: theme.palette.primary.light,
            '& svg': {
              stroke: theme.palette.primary.light
            }
          },
          '& .MuiChip-label': {
            lineHeight: 0
          }
        }}
        label={<IconSettings stroke={1.5} size="24px" color={theme.palette.primary.main} />}
        variant="outlined"
        ref={anchorRef}
        aria-controls={open ? 'menu-list-grow' : undefined}
        aria-haspopup="true"
        onClick={handleToggle}
        color="primary"
      />

      <Popper
        placement="bottom"
        open={open}
        anchorEl={anchorRef.current}
        role={undefined}
        transition
        disablePortal
        popperOptions={{
          modifiers: [
            {
              name: 'offset',
              options: {
                offset: [0, 14]
              }
            }
          ]
        }}
      >
        {({ TransitionProps }) => (
          <ClickAwayListener onClickAway={handleClose}>
            <Transitions in={open} {...TransitionProps}>
              <Paper>
                {open && (
                  <MainCard
                    border={false}
                    elevation={16}
                    content={false}
                    boxShadow
                    shadow={theme.shadows[16]}
                  >
                    {userState.isLoggedIn && (
                      <Box sx={{ p: 2, pb: 0 }}>
                        <Stack>
                          <Stack direction="row" spacing={0.5} alignItems="center">
                            <Typography variant="h4">User:</Typography>
                            <Typography component="span" variant="h4" sx={{ fontWeight: 400 }}>
                              {userState.profile?.name || 'You are not logged in'}
                            </Typography>
                          </Stack>
                          <Typography variant="subtitle2">
                            Role: {userState.authRole.key}
                          </Typography>
                        </Stack>
                        <Divider />
                      </Box>
                    )}
                    <Box sx={{ p: 2, pt: 0 }}>
                      <List
                        component="nav"
                        sx={{
                          width: '100%',
                          maxWidth: 350,
                          minWidth: 300,
                          backgroundColor: theme.palette.background.paper,
                          borderRadius: '10px',
                          [theme.breakpoints.down('md')]: {
                            minWidth: '100%'
                          },
                          '& .MuiListItemButton-root': {
                            mt: 0.5
                          }
                        }}
                      >
                        {userState.isLoggedIn ? (
                          <>
                            <ListItemButton
                              sx={{ borderRadius: `${borderRadius}px` }}
                              onClick={handleLogout}
                            >
                              <ListItemIcon>
                                <IconLogout stroke={1.5} size="20px" />
                              </ListItemIcon>
                              <ListItemText
                                primary={<Typography variant="body2">Logout</Typography>}
                              />
                            </ListItemButton>
                          </>
                        ) : (
                          <LoginModal />
                        )}
                      </List>
                    </Box>
                  </MainCard>
                )}
              </Paper>
            </Transitions>
          </ClickAwayListener>
        )}
      </Popper>
    </>
  );
};

export default MainMenu;
