import PropTypes from 'prop-types';
import { useEffect, useState } from 'react';

import { useTheme } from '@mui/material/styles';

import ReactApexChart from 'react-apexcharts';

import useConfig from 'hooks/useConfig';
import CircularLoading from 'ui-component/cards/CircularLoading';
import { axiosApi } from 'utils/axios';
import { defaultTypeColor } from 'config/layers/styles';

function stringTruncate(str, num) {
  if (str.length > num || str.length === 18) {
    str = `${str.slice(0, num)}..`;
  }
  return str;
}

const defaultBarChartOptions = (title, id) => ({
  title,
  chart: {
    id
  },
  plotOptions: {
    bar: {
      borderRadius: 4,
      horizontal: true,
      distributed: true
    }
  },
  dataLabels: {
    enabled: false
  }
});

// IMPORTANT!! If 'showLegend' is enabled, 'barColors' MUST be supplied or the chart will not load.
export default function BarChart({ id, url, barColors, chart, showLegend, title }) {
  const theme = useTheme();
  const { navType } = useConfig();
  const [colors, setColors] = useState(null);
  const [series, setSeries] = useState([]);
  const [height, setHeight] = useState(400);
  const [loading, setLoading] = useState(true);
  const [categories, setCategories] = useState([]);
  const [options, setOptions] = useState(defaultBarChartOptions(title, id));

  const { primary } = theme.palette.text;
  const darkLight = theme.palette.dark.light;
  const grey200 = theme.palette.grey[200];
  const primaryMain = theme.palette.primary.main;

  const loadChartData = async () => {
    const { data } = await axiosApi.get(url);
    const sortedData = data?.data?.sort((a, b) => parseInt(b.cnt, 10) - parseInt(a.cnt, 10));
    const barHeight = 30;
    const labelHeight = 85;
    setHeight(sortedData.length * barHeight + labelHeight);
    setCategories(
      sortedData.map((category) => {
        if (chart === 'country') {
          return stringTruncate(category.fieldname, 18);
        }
        return category.fieldname || 'null';
      })
    );
    setSeries([{ data: sortedData.map((category) => category.cnt), name: 'Count' }]);
    if (barColors) {
      const tempColorsList = [];
      sortedData.forEach((field) => {
        const colorObj = barColors.find((bar) => field.fieldname === bar.name);
        if (colorObj) {
          tempColorsList.push(colorObj.color);
        } else {
          tempColorsList.push(defaultTypeColor);
        }
      });
      setColors(tempColorsList);
    }
    setLoading(false);
  };

  useEffect(() => {
    loadChartData();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [url]);

  useEffect(() => {
    setOptions((prevState) => ({
      ...prevState,
      colors: colors || [primaryMain],
      xaxis: {
        labels: {
          style: {
            colors: [primary, primary, primary, primary, primary, primary]
          }
        },
        categories
      },
      legend: {
        show: showLegend,
        position: 'bottom',
        fontSize: '11.2px',
        fontFamily: 'Helvetica, Arial',
        fontWeight: 400,
        formatter(seriesName, opts) {
          return [seriesName, '=', series[0].data[opts.seriesIndex]];
        }
      },
      yaxis: {
        labels: {
          style: {
            colors: [
              primary,
              primary,
              primary,
              primary,
              primary,
              primary,
              primary,
              primary,
              primary,
              primary
            ]
          }
        }
      },
      grid: {
        borderColor: navType === 'dark' ? darkLight + 20 : grey200
      },
      tooltip: {
        theme: navType === 'dark' ? 'dark' : 'light'
      }
    }));
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [categories, colors]);

  if (loading) {
    return <CircularLoading />;
  }

  return <ReactApexChart options={options} series={series} type="bar" height={height} />;
}

BarChart.propTypes = {
  id: PropTypes.string,
  url: PropTypes.string,
  chart: PropTypes.string,
  barColors: PropTypes.array,
  showLegend: PropTypes.bool,
  title: PropTypes.object
};
