import { forwardRef, useState } from 'react';
import NumberFormat from 'react-number-format';
import { useSelector, dispatch } from 'store';
import { openSnackbar } from 'store/slices/snackbar';
import PropTypes from 'prop-types';
import { axiosApi } from 'utils/axios';
import _ from 'lodash';
import { reloadVNFLayer } from 'utils/layerUtils';
import {
  Autocomplete,
  Grid,
  TextField,
  CardContent,
  InputLabel,
  Divider,
  CardActions,
  Button,
  Typography
} from '@mui/material';
import { ContextEnum, setCurrentContext, setSelectedEmitterProps } from 'store/slices/dashboard';

const NumberFormatting = forwardRef((props, ref) => {
  const { onChange, name, ...other } = props;

  return (
    <NumberFormat
      {...other}
      getInputRef={ref}
      onValueChange={(values) => {
        onChange({
          target: {
            name: { name },
            value: values.value
          }
        });
      }}
      thousandSeparator
    />
  );
});

export default function Edit() {
  const filterState = useSelector((state) => state.filters);
  const dashboardState = useSelector((state) => state.dashboard);
  const { selectedEmitterProps } = dashboardState;
  const [selectedEmitterClone, setSelectedEmitterClone] = useState(_.clone(selectedEmitterProps));
  const mapState = useSelector((state) => state.map);
  const { map } = mapState;

  const handleReset = () => {
    setSelectedEmitterClone({
      ...selectedEmitterClone,
      COUNTRY: selectedEmitterProps?.COUNTRY ?? undefined,
      iso: selectedEmitterProps?.iso ?? undefined,
      type: selectedEmitterProps?.type ?? undefined
    });
  };

  const handleCancel = () => {
    handleReset();
    dispatch(setCurrentContext(ContextEnum.Details));
  };

  const updateEmitter = async () => {
    let toastMessage;
    let toastColor;
    try {
      const updateURL = `/features/${
        selectedEmitterClone.layerId === 'multiyear' ? 'catalog_multiyear_vnf' : 'vnf'
      }/${selectedEmitterClone.id}`;
      const res = await axiosApi.put(updateURL, selectedEmitterClone);
      reloadVNFLayer(map, selectedEmitterClone.layerId);
      if (res.status === 200) {
        toastMessage = `IR Emitter #${selectedEmitterClone.id} was successfully updated.`;
        toastColor = 'success';
        dispatch(setCurrentContext(ContextEnum.Details));
        dispatch(setSelectedEmitterProps(selectedEmitterClone));
      } else {
        toastMessage = `An error occurred when updating IR Emitter #${selectedEmitterClone.id}, please try again.`;
        toastColor = 'error';
      }
    } catch (error) {
      toastMessage = error?.message
        ? error?.message
        : `An error occurred when updating IR Emitter #${selectedEmitterClone.id}, please try again.`;
      toastColor = 'error';
    }
    dispatch(
      openSnackbar({
        anchorOrigin: {
          vertical: 'top',
          horizontal: 'center'
        },
        open: true,
        message: toastMessage,
        variant: 'alert',
        alert: {
          color: toastColor
        },
        close: true
      })
    );
  };

  return (
    <>
      <CardContent>
        <Grid container spacing={2} alignItems="center">
          <Grid item xs={12} sm={12}>
            <InputLabel>Type</InputLabel>
            <Autocomplete
              id="type-combo-box"
              options={filterState.typeFilters
                .map((x) => x)
                .sort((a, b) => a.label.localeCompare(b.label))}
              getOptionLabel={(option) => option.label}
              value={
                filterState.typeFilters.find((x) =>
                  x.types.includes(selectedEmitterClone?.type?.toLowerCase())
                ) ?? null
              }
              onChange={(e, value) => {
                setSelectedEmitterClone({
                  ...selectedEmitterClone,
                  type: value?.label.toLowerCase() ?? null
                });
              }}
              renderInput={(params) => <TextField {...params} placeholder="Enter Type" />}
            />
          </Grid>
          <Grid item xs={12} sm={12}>
            <InputLabel>Country</InputLabel>
            <Autocomplete
              id="country-combo-box"
              options={filterState.countries}
              getOptionLabel={(option) => option.name}
              value={
                filterState.countries.find((x) => x.alpha3 === selectedEmitterClone?.iso) ?? null
              }
              onChange={(e, value) =>
                setSelectedEmitterClone({
                  ...selectedEmitterClone,
                  iso: value?.alpha3 ?? null,
                  COUNTRY: value?.name ?? null
                })
              }
              renderInput={(params) => <TextField {...params} placeholder="Enter Country" />}
            />
          </Grid>
          <Grid item xs={12} sm={6}>
            <Typography variant="subtitle2">Latitude</Typography>
            <Typography variant="body2">{selectedEmitterClone?.lat?.toFixed(6)}</Typography>
          </Grid>
          <Grid item xs={12} sm={6}>
            <Typography variant="subtitle2">Longitude</Typography>
            <Typography variant="body2">{selectedEmitterClone?.lon?.toFixed(6)}</Typography>
          </Grid>
          <Grid item xs={12} sm={4}>
            <Typography variant="subtitle2">Temperature</Typography>
            <Typography variant="body2">{selectedEmitterClone?.temp_BB ?? 'unknown'}</Typography>
          </Grid>
          <Grid item xs={12} sm={4}>
            <Typography variant="subtitle2">Detection Frequency</Typography>
            <Typography variant="body2">{selectedEmitterClone?.pct ?? 'unknown'}</Typography>
          </Grid>
          <Grid item xs={12} sm={4}>
            <Typography variant="subtitle2">ID</Typography>
            <Typography variant="body2">{selectedEmitterClone?.id}</Typography>
          </Grid>
        </Grid>
      </CardContent>
      <Divider />
      <CardActions>
        <Grid container alignItems="center" justifyContent="space-between" spacing={2}>
          <Grid item>
            <Grid container alignItems="center" justifyContent="flex-end" spacing={2}>
              <Grid item>
                <Button variant="contained" color="error" onClick={() => updateEmitter()}>
                  Save
                </Button>
              </Grid>
              <Grid item>
                <Button variant="outlined" onClick={handleReset}>
                  Reset
                </Button>
              </Grid>
              <Grid item>
                <Button variant="outlined" onClick={handleCancel}>
                  Cancel
                </Button>
              </Grid>
            </Grid>
          </Grid>
        </Grid>
      </CardActions>
    </>
  );
}

NumberFormatting.propTypes = {
  onChange: PropTypes.func,
  name: PropTypes.any
};
