import { useEffect, useState } from 'react';
import {
  Switch,
  FormControlLabel,
  ToggleButton,
  ToggleButtonGroup,
  Typography,
  useTheme
} from '@mui/material';

import { dispatch, useSelector } from 'store';
import { toggleIsRadiant, setLayerStyle, LayerStylesEnum } from 'store/slices/layers';
import { typeStyleColors } from 'config/layers/styles';
import BarChart from './BarChart';

import multiYearHistogram from 'assets/images/Multiyear_T_histograms_by_type.png';
import { filterStyleColors } from 'utils/layerUtils';
import { layerTypes } from 'config/layers/viirsMultiYearConfig';

const typeBaseUrl = '/facets/catalog_multiyear/type?orderBy=type&orderDir=desc';
const countryBaseUrl = '/facets/catalog_multiyear/country?limit=20&orderBycountry&orderDir=desc';

export default function Overview() {
  const theme = useTheme();
  const [typeUrl, setTypeUrl] = useState(typeBaseUrl);
  const [countryUrl, setCountryUrl] = useState(countryBaseUrl);
  const layerState = useSelector((state) => state.layers);
  const dashboardState = useSelector((state) => state.dashboard);
  const { selectionToolCoordinates } = dashboardState;

  const barChartColors = filterStyleColors(typeStyleColors, layerTypes);

  const titleStyle = {
    fontSize: '12px',
    fontWeight: 'bold',
    color: theme.palette.secondary.main
  };

  const headerStyle = {
    fontWeight: 'bold',
    color: titleStyle.color,
    paddingBottom: '10px'
  };

  const handleChangeStyle = (event, newStyle) => {
    if (newStyle) {
      dispatch(setLayerStyle(newStyle));
    }
  };

  useEffect(() => {
    if (selectionToolCoordinates) {
      setTypeUrl(`${typeBaseUrl}&filter=${selectionToolCoordinates}`);
      setCountryUrl(`${countryBaseUrl}&filter=${selectionToolCoordinates}`);
    }
  }, [selectionToolCoordinates]);

  return (
    <div style={{ padding: '0 10px 0 10px' }}>
      <Typography varient="subtitle1">Visualize by:</Typography>
      <ToggleButtonGroup
        value={layerState.style}
        onChange={handleChangeStyle}
        exclusive
        aria-label="text formatting"
        color="secondary"
        fullWidth
        style={{ marginTop: '15px', marginBottom: '15px' }}
      >
        <ToggleButton value={LayerStylesEnum.Type}>Type</ToggleButton>
        <ToggleButton value={LayerStylesEnum.Temp}>Temperature</ToggleButton>
      </ToggleButtonGroup>
      <FormControlLabel
        control={
          <Switch
            checked={layerState.isRadiant}
            color="primary"
            onChange={() => dispatch(toggleIsRadiant())}
          />
        }
        label="Radiant Heat"
      />
      <Typography variant="subtitle1" color="inherit" style={{ paddingBottom: '10px' }}>
        The VIIRS Nightfire (VNF) product utilizes the Visible Infrared Imaging Radiometer Suite
        (VIIRS) satellite instrument to observe sources of infrared (IR) emissions at night,
        including gas flares, steel mills, wildfires, and other industrial and natural high
        temperature sources. Using all the VIIRS bands that collect data at night, Planck curve
        fitting is performed to estimate the temperature and radiant heat of IR emission detections.
        Many of these detections are from persistent sources such as gas flares or volcanoes where
        the detections regularly occur in the same location. For gas flares, the volume of flared
        gas is calculated using a calibration converting radiant heat to flared gas flow rate. The
        2020 Global Gas Flare Catalog is a catalog of all persistent gas flaring sites that were
        active for some portion of 2019-2020, which were determined partially by filtering for for
        for flaring temperatures. The Multiyear 2012-2021 Global IR Emitter Catalog is a catalog of
        all persistent IR emitters around the globe, regardless of temperature, that were active for
        some portion of 2012-2021. For more details, see the{' '}
        <a href="https://eogdata.mines.edu/products/vnf/" target="_blank" rel="noreferrer">
          EOG VNF site.
        </a>
      </Typography>
      <Typography variant="h5" style={headerStyle}>
        Total Number of Emitters by Type and Country (multiyear)
      </Typography>
      <BarChart
        id="Type"
        chart="type"
        url={typeUrl}
        barColors={barChartColors}
        title={{
          text: 'Totals by Type',
          align: 'left',
          floating: true,
          style: titleStyle
        }}
      />
      <BarChart
        id="Country"
        chart="country"
        url={countryUrl}
        title={{
          text: 'Totals by Country',
          align: 'left',
          floating: true,
          style: titleStyle
        }}
      />
      <Typography variant="h5" style={headerStyle}>
        Temperature Histograms by Type (multiyear)
      </Typography>
      <img src={multiYearHistogram} alt="multi year histogram" style={{ width: '100%' }} />
    </div>
  );
}
