import { useEffect, useRef } from 'react';

import styled from '@emotion/styled';

import maplibregl from 'maplibre-gl';
import MapboxDraw from '@mapbox/mapbox-gl-draw';

// import legend from './legend';
import { useSelector, useDispatch } from 'store';
import {
  ContextEnum,
  setCurrentContext,
  setSelectedEmitterProps,
  setSelectionToolCoordinates
} from 'store/slices/dashboard';
import { setMap, setTileStyle, setMapTilerStyle, setInitialized } from 'store/slices/map';
import { generateLayerConfig, getStyle } from 'utils/layerUtils';
import { setFilters } from 'utils/filterUtils';
import mapboxDrawStyles from 'config/mapboxDrawStyles.json';

import 'maplibre-gl/dist/maplibre-gl.css';
import '@mapbox/mapbox-gl-draw/dist/mapbox-gl-draw.css';
// import 'mapboxgl-legend/dist/style.css';
import isAuthorized from 'utils/auth';
import { AuthRoles } from 'store/slices/user';

const MapContainer = styled.div`
  width: 100%;
  height: 100%;
`;

export default function MapComponent() {
  const dispatch = useDispatch();
  const backgroundsState = useSelector((state) => state.backgrounds);
  const filterState = useSelector((state) => state.filters);
  const layerState = useSelector((state) => state.layers);
  const mapState = useSelector((state) => state.map);
  const mapContainerRef = useRef();
  const userState = useSelector((state) => state.user);

  const { customAttribution, initialized, map, mapStyle, lat, lon, zoom } = mapState;
  const { isRadiant, layers } = layerState;
  const { token } = userState;

  const draw = new MapboxDraw({
    displayControlsDefault: false,
    controls: {
      point: true,
      polygon: true,
      trash: true
    },
    styles: mapboxDrawStyles
  });

  if (process.env.REACT_APP_MAPTILER_API_KEY == null) {
    throw new Error('You have to configure env REACT_APP_MAPTILER_API_KEY, see README');
  }

  useEffect(() => {
    dispatch(setSelectedEmitterProps(null));
    dispatch(setCurrentContext(ContextEnum.Overview));
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  // this effect is used to set the map style based on the type of background selected
  useEffect(() => {
    const { url, isTile } = backgroundsState.bgSelected;
    if (url) {
      if (isTile) {
        dispatch(setTileStyle(url));
      } else {
        dispatch(setMapTilerStyle(url, process.env.REACT_APP_MAPTILER_API_KEY));
      }
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [backgroundsState.bgSelected]);

  // this effect is used to create the map
  useEffect(() => {
    if (mapContainerRef?.current && mapStyle && lon && lat) {
      dispatch(
        setMap(
          new maplibregl.Map({
            attributionControl: true,
            center: [lon, lat],
            customAttribution,
            container: mapContainerRef.current,
            style: mapStyle,
            zoom
          })
        )
      );
      dispatch(setInitialized(false));
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [mapContainerRef, mapStyle]);

  // this effect is to handle adding the legend, controls, and miscellaneous map features
  useEffect(() => {
    if (map && !initialized) {
      map.addControl(draw);
      map.addControl(new maplibregl.NavigationControl(), 'top-right');
      // map.addControl(legend(), 'top-left');
      map.boxZoom.enable();
      dispatch(setInitialized(true));
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [map, initialized]);

  // this effect is to handle adding layers
  useEffect(() => {
    if (map) {
      const callbackObject = {
        click: (properties) => {
          if (properties) {
            dispatch(setSelectedEmitterProps(properties));
            dispatch(setCurrentContext(ContextEnum.Details));
          } else {
            dispatch(setCurrentContext(ContextEnum.Overview));
            dispatch(setSelectedEmitterProps(null));
          }
        }
      };
      layers.forEach((layer) => {
        const layerData = map.getLayer(layer.id);
        if (layer.enabled) {
          if (layerData === undefined) {
            Object.keys(layer.on).forEach((key) => {
              map.on(
                key,
                ...layer.on[key](
                  map,
                  draw,
                  callbackObject[key],
                  { layerState, filterState },
                  dispatch,
                  setSelectionToolCoordinates
                )
              );
            });
          }
        }
      });
      // hack for calling the setFilters only a single time after loading to apply the url query param rangefilter
      let idleFlag = true;
      map.on('idle', () => {
        if (idleFlag) {
          setFilters(filterState, layers, map);
        }
        idleFlag = false;
      });
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [map]);

  // this effect is used to update the map when the styles are changed
  useEffect(() => {
    if (map) {
      layers.forEach((layer) => {
        const { config } = layer;
        const layerData = map.getLayer(layer.id);
        const { circleRadius, circleColor } = getStyle(layerState.style, config, isRadiant);
        if (layer.enabled) {
          if (layerData !== undefined) {
            layer.config?.sublayers?.forEach((sublayer) => {
              if (map.getLayer(sublayer.id)) {
                map.removeLayer(sublayer.id);
              }
            });
            map.removeLayer(config.id);
          }
          if (map.getSource(config.id)) {
            map.addLayer(
              generateLayerConfig(
                config.id,
                config.layerType,
                config.paint(circleRadius, circleColor),
                config.id,
                config.sourceLayer,
                config.metadata || {}
              )
            );
            if (layer.sublayers?.length > 0) {
              layer.config?.sublayers?.forEach((sublayer) => {
                map.addLayer(
                  generateLayerConfig(
                    sublayer.id,
                    sublayer.layerType,
                    sublayer.paint(),
                    layer.id,
                    sublayer.sourceLayer,
                    sublayer.metadata || {}
                  )
                );
              });
            }
            setFilters(filterState, layers, map);
          }
        }
      });
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [layerState]);

  // this effect is to update the map's filters when they're changed
  useEffect(() => {
    if (map && initialized) {
      setFilters(filterState, layers, map);
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [filterState]);

  // this effect is to update the map's transformRquest with the signed in token when sign in happens
  useEffect(() => {
    if (map && isAuthorized(userState, AuthRoles.Subscriber)) {
      const transform = (url, resourceType) => {
        if (resourceType === 'Tile' && url.startsWith(`${process.env.REACT_APP_API_URL}`)) {
          return {
            url,
            headers: { Authorization: `Bearer ${token}` }
          };
        }
        return {
          url
        };
      };
      map.setTransformRequest(transform.bind(token));
    }
  }, [userState, map, token]);

  return <MapContainer ref={mapContainerRef} />;
}
