import PropTypes from 'prop-types';
import { forwardRef } from 'react';

import { CopyToClipboard } from 'react-copy-to-clipboard';
import {
  Switch,
  FormControlLabel,
  Grid,
  Typography,
  Box,
  Tooltip,
  IconButton
} from '@mui/material';

import { dispatch } from 'store';
import { toggleLayerEnabled } from 'store/slices/layers';
import { openSnackbar } from 'store/slices/snackbar';

import { IconCopy, IconLock, IconLockOpen } from '@tabler/icons';

export default function LayerComponent(props) {
  const { layerId, name, labelText, enabled, locked, tooltipText, tileSource } = props;

  const WrappedIcon = forwardRef((props, ref) => (
    <div {...props} ref={ref}>
      {locked ? <IconLock /> : <IconLockOpen />}
    </div>
  ));

  return (
    <Grid item xs={12}>
      <Typography variant="subtitle1" sx={{ textTransform: 'uppercase' }}>
        {name}
      </Typography>
      <Box sx={{ display: 'flex' }}>
        <Box sx={{ padding: '8px', justifyContent: 'center', alignItems: 'center' }}>
          <Tooltip title={tooltipText}>
            <WrappedIcon />
          </Tooltip>
        </Box>
        <FormControlLabel
          control={
            <Switch
              checked={locked ? false : enabled}
              onChange={() => dispatch(toggleLayerEnabled(layerId))}
              name={layerId}
              color="primary"
              disabled={locked}
            />
          }
          label={labelText}
        />
      </Box>
      <Box style={{ display: 'flex' }}>
        <Typography variant="h6" style={{ marginRight: '10px' }}>
          Tile Source:
        </Typography>
        <Typography variant="body2" style={{ fontSize: '12px' }}>
          {tileSource}
        </Typography>
        <CopyToClipboard
          text={tileSource}
          style={{ borderRadius: '0', padding: '0', marginLeft: '10px' }}
          onCopy={() =>
            dispatch(
              openSnackbar({
                open: true,
                message: 'Tile Source Copied',
                variant: 'alert',
                alert: {
                  color: 'success'
                },
                close: false,
                anchorOrigin: { vertical: 'top', horizontal: 'center' },
                transition: 'SlideDown'
              })
            )
          }
        >
          <Tooltip title="Copy Tile Source">
            <IconButton>
              <IconCopy size={20} />
            </IconButton>
          </Tooltip>
        </CopyToClipboard>
      </Box>
    </Grid>
  );
}

LayerComponent.propTypes = {
  layerId: PropTypes.string.isRequired,
  name: PropTypes.string.isRequired,
  labelText: PropTypes.string.isRequired,
  enabled: PropTypes.bool.isRequired,
  locked: PropTypes.bool.isRequired,
  tooltipText: PropTypes.string,
  tileSource: PropTypes.string
};
